@@ -97,7 +97,7 @@ impl FeaturesManager {
         // gl_ClipDistance is supported by core versions > 1.3 and aren't supported by an es versions without extensions
         check_feature!(CLIP_DISTANCE, 130, 300);
         check_feature!(CULL_DISTANCE, 450, 300);
-        check_feature!(SAMPLE_VARIABLES, 300, 300);
+        check_feature!(SAMPLE_VARIABLES, 400, 300);
 
         // Return an error if there are missing features
         if missing.is_empty() {
@@ -315,6 +315,9 @@ impl<'a, W> Writer<'a, W> {
                             crate::BuiltIn::ClipDistance => {
                                 self.features.request(Features::CLIP_DISTANCE)
                             }
+                            crate::BuiltIn::CullDistance => {
+                                self.features.request(Features::CULL_DISTANCE)
+                            }
                             crate::BuiltIn::SampleIndex => {
                                 self.features.request(Features::SAMPLE_VARIABLES)
                             }
