@@ -106,7 +106,7 @@ impl Version {
 
     /// Checks if the version supports explicit `layout(location=)` qualifiers.
     fn supports_explicit_locations(&self) -> bool {
-        *self >= Version::Embedded(310) || *self >= Version::Desktop(410)
+        *self >= Version::Embedded(300) || *self >= Version::Desktop(410)
     }
 }
 
@@ -194,9 +194,11 @@ struct FunctionCtx<'a> {
     info: &'a FunctionInfo,
     /// The expression arena of the current function being written
     expressions: &'a Arena<Expression>,
+    /// Map of expressions that have associated variable names
+    named_expressions: &'a crate::NamedExpressions,
 }
 
-impl<'a> FunctionCtx<'a> {
+impl<'a> FunctionCtx<'_> {
     /// Helper method that generates a [`NameKey`](crate::proc::NameKey) for a local in the current function
     fn name_key(&self, local: Handle<LocalVariable>) -> NameKey {
         match self.func {
@@ -295,9 +297,6 @@ pub enum Error {
     /// A scalar with an unsupported width was requested
     #[error("A scalar with an unsupported width was requested: {0:?} {1:?}")]
     UnsupportedScalar(ScalarKind, Bytes),
-    /// [`Interpolation::Patch`](crate::Interpolation::Patch) isn't supported
-    #[error("Patch interpolation isn't supported")]
-    PatchInterpolationNotSupported,
     /// A image was used with multiple samplers, this isn't supported
     #[error("A image was used with multiple samplers")]
     ImageMultipleSamplers,
@@ -320,6 +319,7 @@ pub struct Writer<'a, W> {
     // Internal State
     /// Features manager used to store all the needed features and write them
     features: FeaturesManager,
+    namer: Namer,
     /// A map with all the names needed for writing the module
     /// (generated by a [`Namer`](crate::proc::Namer))
     names: FastHashMap<NameKey, String>,
@@ -332,7 +332,7 @@ pub struct Writer<'a, W> {
     /// Used to generate a unique number for blocks
     block_id: IdGenerator,
     /// Set of expressions that have associated temporary variables
-    cached_expressions: FastHashMap<Handle<Expression>, String>,
+    named_expressions: crate::NamedExpressions,
 }
 
 impl<'a, W: Write> Writer<'a, W> {
@@ -363,7 +363,8 @@ impl<'a, W: Write> Writer<'a, W> {
 
         // Generate a map with names required to write the module
         let mut names = FastHashMap::default();
-        Namer::default().reset(module, keywords::RESERVED_KEYWORDS, &mut names);
+        let mut namer = Namer::default();
+        namer.reset(module, keywords::RESERVED_KEYWORDS, &["gl_"], &mut names);
 
         // Build the instance
         let mut this = Self {
@@ -371,7 +372,7 @@ impl<'a, W: Write> Writer<'a, W> {
             info,
             out,
             options,
-
+            namer,
             features: FeaturesManager::new(),
             names,
             reflection_names: FastHashMap::default(),
@@ -379,7 +380,7 @@ impl<'a, W: Write> Writer<'a, W> {
             entry_point_idx: ep_idx as u16,
 
             block_id: IdGenerator::default(),
-            cached_expressions: FastHashMap::default(),
+            named_expressions: crate::NamedExpressions::default(),
         };
 
         // Find all features required to print this module
@@ -495,15 +496,8 @@ impl<'a, W: Write> Writer<'a, W> {
                         write!(self.out, "layout({}) ", glsl_storage_format(format))?;
                     }
 
-                    // Write the storage access modifier
-                    //
-                    // glsl allows adding both `readonly` and `writeonly` but this means that
-                    // they can only be used to query information about the image which isn't what
-                    // we want here so when storage access is both `LOAD` and `STORE` add no modifiers
-                    if global.storage_access == StorageAccess::LOAD {
-                        write!(self.out, "readonly ")?;
-                    } else if global.storage_access == StorageAccess::STORE {
-                        write!(self.out, "writeonly ")?;
+                    if let Some(storage_access) = glsl_storage_access(global.storage_access) {
+                        write!(self.out, "{} ", storage_access)?;
                     }
 
                     // All images in glsl are `uniform`
@@ -527,7 +521,11 @@ impl<'a, W: Write> Writer<'a, W> {
                 // glsl has no concept of samplers so we just ignore it
                 TypeInner::Sampler { .. } => continue,
                 // All other globals are written by `write_global`
-                _ => self.write_global(handle, global)?,
+                _ => {
+                    self.write_global(handle, global)?;
+                    // Add a newline (only for readability)
+                    writeln!(self.out)?;
+                }
             }
         }
 
@@ -571,6 +569,27 @@ impl<'a, W: Write> Writer<'a, W> {
         self.collect_reflection_info()
     }
 
+    fn write_array_size(&mut self, size: ArraySize) -> BackendResult {
+        write!(self.out, "[")?;
+
+        // Write the array size
+        // Writes nothing if `ArraySize::Dynamic`
+        // Panics if `ArraySize::Constant` has a constant that isn't an uint
+        match size {
+            ArraySize::Constant(const_handle) => match self.module.constants[const_handle].inner {
+                ConstantInner::Scalar {
+                    width: _,
+                    value: ScalarValue::Uint(size),
+                } => write!(self.out, "{}", size)?,
+                _ => unreachable!(),
+            },
+            ArraySize::Dynamic => (),
+        }
+
+        write!(self.out, "]")?;
+        Ok(())
+    }
+
     /// Helper method used to write value types
     ///
     /// # Notes
@@ -622,27 +641,7 @@ impl<'a, W: Write> Writer<'a, W> {
             // GLSL arrays are written as `type name[size]`
             // Current code is written arrays only as `[size]`
             // Base `type` and `name` should be written outside
-            TypeInner::Array { base: _, size, .. } => {
-                write!(self.out, "[")?;
-
-                // Write the array size
-                // Writes nothing if `ArraySize::Dynamic`
-                // Panics if `ArraySize::Constant` has a constant that isn't an uint
-                match size {
-                    ArraySize::Constant(const_handle) => {
-                        match self.module.constants[const_handle].inner {
-                            ConstantInner::Scalar {
-                                width: _,
-                                value: ScalarValue::Uint(size),
-                            } => write!(self.out, "{}", size)?,
-                            _ => unreachable!(),
-                        }
-                    }
-                    ArraySize::Dynamic => (),
-                }
-
-                write!(self.out, "]")?
-            }
+            TypeInner::Array { size, .. } => self.write_array_size(size)?,
             // Panic if either Image, Sampler, Pointer, or a Struct is being written
             //
             // Write all variants instead of `_` so that if new variants are added a
@@ -670,7 +669,7 @@ impl<'a, W: Write> Writer<'a, W> {
             // glsl has no pointer types so just write types as normal and loads are skipped
             TypeInner::Pointer { base, .. } => self.write_type(base),
             TypeInner::Struct {
-                level: crate::StructLevel::Root,
+                top_level: true,
                 ref members,
                 span: _,
             } => self.write_struct(true, ty, members),
@@ -681,6 +680,8 @@ impl<'a, W: Write> Writer<'a, W> {
                 write!(self.out, "{}", name)?;
                 Ok(())
             }
+            // glsl array has the size separated from the base type
+            TypeInner::Array { base, .. } => self.write_type(base),
             ref other => self.write_value_type(other),
         }
     }
@@ -741,20 +742,20 @@ impl<'a, W: Write> Writer<'a, W> {
         handle: Handle<GlobalVariable>,
         global: &GlobalVariable,
     ) -> BackendResult {
-        // Write the storage access modifier
-        //
-        // glsl allows adding both `readonly` and `writeonly` but this means that
-        // they can only be used to query information about the resource which isn't what
-        // we want here so when storage access is both `LOAD` and `STORE` add no modifiers
-        if global.storage_access == StorageAccess::LOAD {
-            write!(self.out, "readonly ")?;
-        } else if global.storage_access == StorageAccess::STORE {
-            write!(self.out, "writeonly ")?;
+        if let Some(storage_access) = glsl_storage_access(global.storage_access) {
+            write!(self.out, "{} ", storage_access)?;
         }
 
         // Write the storage class
         // Trailing space is important
-        write!(self.out, "{} ", glsl_storage_class(global.class))?;
+        if let Some(storage_class) = glsl_storage_class(global.class) {
+            write!(self.out, "{} ", storage_class)?;
+        } else if let TypeInner::Struct {
+            top_level: true, ..
+        } = self.module.types[global.ty].inner
+        {
+            write!(self.out, "struct ")?;
+        }
 
         // Write the type
         // `write_type` adds no leading or trailing spaces
@@ -762,8 +763,15 @@ impl<'a, W: Write> Writer<'a, W> {
 
         // Finally write the global name and end the global with a `;` and a newline
         // Leading space is important
-        writeln!(self.out, " {};", self.get_global_name(handle, global))?;
-        writeln!(self.out)?;
+        let global_name = self.get_global_name(handle, global);
+        write!(self.out, " {}", global_name)?;
+        if let TypeInner::Array { size, .. } = self.module.types[global.ty].inner {
+            self.write_array_size(size)?;
+        }
+        if let Some(default_value) = zero_init_value_str(&self.module.types[global.ty].inner) {
+            write!(self.out, " = {}", default_value)?;
+        };
+        writeln!(self.out, ";")?;
 
         Ok(())
     }
@@ -822,7 +830,9 @@ impl<'a, W: Write> Writer<'a, W> {
                 }
 
                 // Write the storage class
-                if self.options.version.supports_explicit_locations() {
+                if !emit_interpolation_and_auxiliary
+                    && self.options.version.supports_explicit_locations()
+                {
                     write!(self.out, "layout(location = {}) ", location)?;
                 }
 
@@ -879,9 +889,10 @@ impl<'a, W: Write> Writer<'a, W> {
             func: ty,
             info,
             expressions: &func.expressions,
+            named_expressions: &func.named_expressions,
         };
 
-        self.cached_expressions.clear();
+        self.named_expressions.clear();
 
         // Write the function header
         //
@@ -1117,7 +1128,11 @@ impl<'a, W: Write> Writer<'a, W> {
             write!(self.out, "{}", INDENT)?;
 
             match self.module.types[member.ty].inner {
-                TypeInner::Array { base, .. } => {
+                TypeInner::Array {
+                    base,
+                    size,
+                    stride: _,
+                } => {
                     // GLSL arrays are written as `type name[size]`
                     let ty_name = match self.module.types[base].inner {
                         // Write scalar type by backend so as not to depend on the front-end implementation
@@ -1134,7 +1149,7 @@ impl<'a, W: Write> Writer<'a, W> {
                         &self.names[&NameKey::StructMember(handle, idx as u32)]
                     )?;
                     // Write [size]
-                    self.write_type(member.ty)?;
+                    self.write_array_size(size)?;
                     // Newline is important
                     writeln!(self.out, ";")?;
                 }
@@ -1180,30 +1195,24 @@ impl<'a, W: Write> Writer<'a, W> {
             // This is where we can generate intermediate constants for some expression types.
             Statement::Emit(ref range) => {
                 for handle in range.clone() {
-                    let min_ref_count = ctx.expressions[handle].bake_ref_count();
-                    if min_ref_count <= ctx.info[handle].ref_count {
-                        write!(self.out, "{}", INDENT.repeat(indent))?;
-                        match ctx.info[handle].ty {
-                            TypeResolution::Handle(ty_handle) => {
-                                match self.module.types[ty_handle].inner {
-                                    TypeInner::Struct { .. } => {
-                                        let ty_name = &self.names[&NameKey::Type(ty_handle)];
-                                        write!(self.out, "{}", ty_name)?;
-                                    }
-                                    _ => {
-                                        self.write_type(ty_handle)?;
-                                    }
-                                }
-                            }
-                            TypeResolution::Value(ref inner) => {
-                                self.write_value_type(inner)?;
-                            }
+                    let expr_name = if let Some(name) = ctx.named_expressions.get(&handle) {
+                        // Front end provides names for all variables at the start of writing.
+                        // But we write them to step by step. We need to recache them
+                        // Otherwise, we could accidentally write variable name instead of full expression.
+                        // Also, we use sanitized names! It defense backend from generating variable with name from reserved keywords.
+                        Some(self.namer.call_unique(name))
+                    } else {
+                        let min_ref_count = ctx.expressions[handle].bake_ref_count();
+                        if min_ref_count <= ctx.info[handle].ref_count {
+                            Some(format!("_expr{}", handle.index()))
+                        } else {
+                            None
                         }
-                        let name = format!("_expr{}", handle.index());
-                        write!(self.out, " {} = ", name)?;
-                        self.write_expr(handle, ctx)?;
-                        writeln!(self.out, ";")?;
-                        self.cached_expressions.insert(handle, name);
+                    };
+
+                    if let Some(name) = expr_name {
+                        write!(self.out, "{}", INDENT.repeat(indent))?;
+                        self.write_named_expr(handle, name, ctx)?;
                     }
                 }
             }
@@ -1233,7 +1242,7 @@ impl<'a, W: Write> Writer<'a, W> {
                 ref reject,
             } => {
                 write!(self.out, "{}", INDENT.repeat(indent))?;
-                write!(self.out, "if(")?;
+                write!(self.out, "if (")?;
                 self.write_expr(condition, ctx)?;
                 writeln!(self.out, ") {{")?;
 
@@ -1296,7 +1305,7 @@ impl<'a, W: Write> Writer<'a, W> {
                     }
 
                     // Write `break;` if the block isn't fallthrough
-                    if case.fall_through {
+                    if !case.fall_through {
                         writeln!(self.out, "{}break;", INDENT.repeat(indent + 2))?;
                     }
                 }
@@ -1364,17 +1373,50 @@ impl<'a, W: Write> Writer<'a, W> {
                             let value = value.unwrap();
                             match self.module.types[result.ty].inner {
                                 crate::TypeInner::Struct { ref members, .. } => {
+                                    let (mut is_temp_struct_used, mut return_struct) = (false, "");
+                                    if let Expression::Compose { .. } = ctx.expressions[value] {
+                                        is_temp_struct_used = true;
+                                        return_struct = "_tmp_return";
+                                        write!(
+                                            self.out,
+                                            "{} {} = ",
+                                            &self.names[&NameKey::Type(result.ty)],
+                                            return_struct
+                                        )?;
+                                        self.write_expr(value, ctx)?;
+                                        writeln!(self.out, ";")?;
+                                        write!(self.out, "{}", INDENT.repeat(indent))?;
+                                    }
                                     for (index, member) in members.iter().enumerate() {
+                                        // TODO: handle builtin in better way
+                                        if let Some(Binding::BuiltIn(builtin)) = member.binding {
+                                            match builtin {
+                                                crate::BuiltIn::ClipDistance
+                                                | crate::BuiltIn::CullDistance
+                                                | crate::BuiltIn::PointSize => {
+                                                    if self.options.version.is_es() {
+                                                        continue;
+                                                    }
+                                                }
+                                                _ => {}
+                                            }
+                                        }
+
                                         let varying_name = VaryingName {
                                             binding: member.binding.as_ref().unwrap(),
                                             stage: ep.stage,
                                             output: true,
                                         };
+                                        let field_name = self.names
+                                            [&NameKey::StructMember(result.ty, index as u32)]
+                                            .clone();
                                         write!(self.out, "{} = ", varying_name)?;
-                                        self.write_expr(value, ctx)?;
-                                        let field_name = &self.names
-                                            [&NameKey::StructMember(result.ty, index as u32)];
-                                        writeln!(self.out, ".{};", field_name)?;
+
+                                        if !is_temp_struct_used {
+                                            self.write_expr(value, ctx)?;
+                                        }
+
+                                        writeln!(self.out, "{}.{};", return_struct, &field_name)?;
                                         write!(self.out, "{}", INDENT.repeat(indent))?;
                                     }
                                 }
@@ -1398,9 +1440,14 @@ impl<'a, W: Write> Writer<'a, W> {
             // This is one of the places were glsl adds to the syntax of C in this case the discard
             // keyword which ceases all further processing in a fragment shader, it's called OpKill
             // in spir-v that's why it's called `Statement::Kill`
-            Statement::Kill => {
-                write!(self.out, "{}", INDENT.repeat(indent))?;
-                writeln!(self.out, "discard;")?
+            Statement::Kill => writeln!(self.out, "{}discard;", INDENT.repeat(indent))?,
+            // Issue an execution or a memory barrier.
+            Statement::Barrier(flags) => {
+                if flags.is_empty() {
+                    writeln!(self.out, "{}barrier();", INDENT.repeat(indent))?;
+                } else {
+                    writeln!(self.out, "{}groupMemoryBarrier();", INDENT.repeat(indent))?;
+                }
             }
             // Stores in glsl are just variable assignments written as `pointer = value;`
             Statement::Store { pointer, value } => {
@@ -1444,7 +1491,7 @@ impl<'a, W: Write> Writer<'a, W> {
                     let result = self.module.functions[function].result.as_ref().unwrap();
                     self.write_type(result.ty)?;
                     write!(self.out, " {} = ", name)?;
-                    self.cached_expressions.insert(expr, name);
+                    self.named_expressions.insert(expr, name);
                 }
                 write!(self.out, "{}(", &self.names[&NameKey::Function(function)])?;
                 self.write_slice(arguments, |this, _, arg| this.write_expr(*arg, ctx))?;
@@ -1460,10 +1507,11 @@ impl<'a, W: Write> Writer<'a, W> {
     /// # Notes
     /// Doesn't add any newlines or leading/trailing spaces
     fn write_expr(&mut self, expr: Handle<Expression>, ctx: &FunctionCtx<'_>) -> BackendResult {
-        if let Some(name) = self.cached_expressions.get(&expr) {
+        if let Some(name) = self.named_expressions.get(&expr) {
             write!(self.out, "{}", name)?;
             return Ok(());
         }
+
         match ctx.expressions[expr] {
             // `Access` is applied to arrays, vectors and matrices and is written as indexing
             Expression::Access { base, index } => {
@@ -1988,45 +2036,39 @@ impl<'a, W: Write> Writer<'a, W> {
                 convert,
             } => {
                 let inner = ctx.info[expr].ty.inner_with(&self.module.types);
-                if convert {
-                    // this is similar to `write_type`, but with the target kind
-                    match *inner {
-                        TypeInner::Scalar { kind: _, width } => {
-                            write!(self.out, "{}", glsl_scalar(target_kind, width)?.full)?
-                        }
-                        TypeInner::Vector {
-                            size,
-                            kind: _,
-                            width,
-                        } => write!(
-                            self.out,
-                            "{}vec{}",
-                            glsl_scalar(target_kind, width)?.prefix,
-                            size as u8
-                        )?,
-                        ref other => unreachable!("unexpected cast of {:?}", other),
-                    }
-                } else {
-                    let source_kind = inner.scalar_kind().unwrap();
-                    write!(
-                        self.out,
-                        "{}",
-                        match (source_kind, target_kind) {
-                            (ScalarKind::Float, ScalarKind::Sint) => "floatBitsToInt",
-                            (ScalarKind::Float, ScalarKind::Uint) => "floatBitsToUInt",
-                            (ScalarKind::Sint, ScalarKind::Float) => "intBitsToFloat",
-                            (ScalarKind::Uint, ScalarKind::Float) => "uintBitsToFloat",
-                            // There is no way to bitcast between Uint/Sint in glsl. Use constructor conversion
-                            (ScalarKind::Uint, ScalarKind::Sint) => "int",
-                            (ScalarKind::Sint, ScalarKind::Uint) => "uint",
-                            _ => {
-                                return Err(Error::Custom(format!(
-                                    "Cannot bitcast {:?} to {:?}",
-                                    source_kind, target_kind
-                                )));
+                match convert {
+                    Some(width) => {
+                        // this is similar to `write_type`, but with the target kind
+                        let scalar = glsl_scalar(target_kind, width)?;
+                        match *inner {
+                            TypeInner::Vector { size, .. } => {
+                                write!(self.out, "{}vec{}", scalar.prefix, size as u8)?
                             }
+                            _ => write!(self.out, "{}", scalar.full)?,
                         }
-                    )?;
+                    }
+                    None => {
+                        let source_kind = inner.scalar_kind().unwrap();
+                        write!(
+                            self.out,
+                            "{}",
+                            match (source_kind, target_kind) {
+                                (ScalarKind::Float, ScalarKind::Sint) => "floatBitsToInt",
+                                (ScalarKind::Float, ScalarKind::Uint) => "floatBitsToUInt",
+                                (ScalarKind::Sint, ScalarKind::Float) => "intBitsToFloat",
+                                (ScalarKind::Uint, ScalarKind::Float) => "uintBitsToFloat",
+                                // There is no way to bitcast between Uint/Sint in glsl. Use constructor conversion
+                                (ScalarKind::Uint, ScalarKind::Sint) => "int",
+                                (ScalarKind::Sint, ScalarKind::Uint) => "uint",
+                                _ => {
+                                    return Err(Error::Custom(format!(
+                                        "Cannot bitcast {:?} to {:?}",
+                                        source_kind, target_kind
+                                    )));
+                                }
+                            }
+                        )?;
+                    }
                 }
 
                 write!(self.out, "(")?;
@@ -2073,6 +2115,45 @@ impl<'a, W: Write> Writer<'a, W> {
         Ok(())
     }
 
+    fn write_named_expr(
+        &mut self,
+        handle: Handle<Expression>,
+        name: String,
+        ctx: &FunctionCtx,
+    ) -> BackendResult {
+        match ctx.info[handle].ty {
+            TypeResolution::Handle(ty_handle) => match self.module.types[ty_handle].inner {
+                TypeInner::Struct { .. } => {
+                    let ty_name = &self.names[&NameKey::Type(ty_handle)];
+                    write!(self.out, "{}", ty_name)?;
+                }
+                _ => {
+                    self.write_type(ty_handle)?;
+                }
+            },
+            TypeResolution::Value(ref inner) => {
+                self.write_value_type(inner)?;
+            }
+        }
+
+        let base_ty_res = &ctx.info[handle].ty;
+        let resolved = base_ty_res.inner_with(&self.module.types);
+
+        // If rhs is a array type, we should write temp variable as a dynamic array
+        let array_str = if let TypeInner::Array { .. } = *resolved {
+            "[]"
+        } else {
+            ""
+        };
+
+        write!(self.out, " {}{} = ", name, array_str)?;
+        self.write_expr(handle, ctx)?;
+        writeln!(self.out, ";")?;
+        self.named_expressions.insert(handle, name);
+
+        Ok(())
+    }
+
     /// Helper method used to produce the reflection info that's returned to the user
     ///
     /// It takes an iterator of [`Function`](crate::Function) references instead of
@@ -2185,6 +2266,7 @@ fn glsl_built_in(built_in: BuiltIn, output: bool) -> &'static str {
         BuiltIn::BaseInstance => "uint(gl_BaseInstance)",
         BuiltIn::BaseVertex => "uint(gl_BaseVertex)",
         BuiltIn::ClipDistance => "gl_ClipDistance",
+        BuiltIn::CullDistance => "gl_CullDistance",
         BuiltIn::InstanceIndex => "uint(gl_InstanceID)",
         BuiltIn::PointSize => "gl_PointSize",
         BuiltIn::VertexIndex => "uint(gl_VertexID)",
@@ -2209,15 +2291,15 @@ fn glsl_built_in(built_in: BuiltIn, output: bool) -> &'static str {
 }
 
 /// Helper function that returns the string corresponding to the storage class
-fn glsl_storage_class(class: StorageClass) -> &'static str {
+fn glsl_storage_class(class: StorageClass) -> Option<&'static str> {
     match class {
-        StorageClass::Function => "",
-        StorageClass::Private => "",
-        StorageClass::Storage => "buffer",
-        StorageClass::Uniform => "uniform",
-        StorageClass::Handle => "uniform",
-        StorageClass::WorkGroup => "shared",
-        StorageClass::PushConstant => "",
+        StorageClass::Function => None,
+        StorageClass::Private => None,
+        StorageClass::Storage => Some("buffer"),
+        StorageClass::Uniform => Some("uniform"),
+        StorageClass::Handle => Some("uniform"),
+        StorageClass::WorkGroup => Some("shared"),
+        StorageClass::PushConstant => None,
     }
 }
 
@@ -2286,3 +2368,41 @@ fn glsl_storage_format(format: StorageFormat) -> &'static str {
         StorageFormat::Rgba32Float => "rgba32f",
     }
 }
+
+/// Helper function that return the glsl storage access string of [`StorageAccess`](crate::StorageAccess)
+///
+/// glsl allows adding both `readonly` and `writeonly` but this means that
+/// they can only be used to query information about the resource which isn't what
+/// we want here so when storage access is both `LOAD` and `STORE` add no modifiers
+fn glsl_storage_access(storage_access: StorageAccess) -> Option<&'static str> {
+    if storage_access == StorageAccess::LOAD {
+        Some("readonly")
+    } else if storage_access == StorageAccess::STORE {
+        Some("writeonly")
+    } else {
+        None
+    }
+}
+
+/// Helper function that return string with default zero initialization for supported types
+fn zero_init_value_str(inner: &TypeInner) -> Option<String> {
+    match *inner {
+        TypeInner::Scalar { kind, .. } => match kind {
+            ScalarKind::Bool => Some(String::from("false")),
+            _ => Some(String::from("0")),
+        },
+        TypeInner::Vector { size, kind, width } => {
+            if let Ok(scalar_string) = glsl_scalar(kind, width) {
+                let vec_type = format!("{}vec{}", scalar_string.prefix, size as u8);
+                match size {
+                    crate::VectorSize::Bi => Some(format!("{}(0, 0)", vec_type)),
+                    crate::VectorSize::Tri => Some(format!("{}(0, 0, 0)", vec_type)),
+                    crate::VectorSize::Quad => Some(format!("{}(0, 0, 0, 0)", vec_type)),
+                }
+            } else {
+                None
+            }
+        }
+        _ => None,
+    }
+}
