@@ -51,6 +51,8 @@ pub enum ConstantSolvingError {
     InvalidBinaryOpArgs,
     #[error("Splat/swizzle type is not registered")]
     DestinationTypeNotFound,
+    #[error("Not implemented: {0}")]
+    NotImplemented(String),
 }
 
 impl<'a> ConstantSolver<'a> {
@@ -152,7 +154,9 @@ impl<'a> ConstantSolver<'a> {
 
                 self.binary_op(op, left_constant, right_constant)
             }
-            Expression::Math { .. } => todo!(),
+            Expression::Math { fun, .. } => {
+                Err(ConstantSolvingError::NotImplemented(format!("{:?}", fun)))
+            }
             Expression::As {
                 convert,
                 expr,
@@ -160,10 +164,9 @@ impl<'a> ConstantSolver<'a> {
             } => {
                 let expr_constant = self.solve(expr)?;
 
-                if convert {
-                    self.cast(expr_constant, kind)
-                } else {
-                    Err(ConstantSolvingError::Bitcast)
+                match convert {
+                    Some(width) => self.cast(expr_constant, kind, width),
+                    None => Err(ConstantSolvingError::Bitcast),
                 }
             }
             Expression::ArrayLength(expr) => {
@@ -239,6 +242,7 @@ impl<'a> ConstantSolver<'a> {
         &mut self,
         constant: Handle<Constant>,
         kind: ScalarKind,
+        target_width: crate::Bytes,
     ) -> Result<Handle<Constant>, ConstantSolvingError> {
         fn inner_cast<A: num_traits::FromPrimitive>(value: ScalarValue) -> A {
             match value {
@@ -253,7 +257,11 @@ impl<'a> ConstantSolver<'a> {
         let mut inner = self.constants[constant].inner.clone();
 
         match inner {
-            ConstantInner::Scalar { ref mut value, .. } => {
+            ConstantInner::Scalar {
+                ref mut value,
+                ref mut width,
+            } => {
+                *width = target_width;
                 *value = match kind {
                     ScalarKind::Sint => ScalarValue::Sint(inner_cast(*value)),
                     ScalarKind::Uint => ScalarValue::Uint(inner_cast(*value)),
@@ -271,7 +279,7 @@ impl<'a> ConstantSolver<'a> {
                 }
 
                 for component in components {
-                    *component = self.cast(*component, kind)?;
+                    *component = self.cast(*component, kind, target_width)?;
                 }
             }
         }
@@ -561,7 +569,7 @@ mod tests {
         let root = expressions.append(Expression::As {
             expr,
             kind: ScalarKind::Bool,
-            convert: true,
+            convert: Some(crate::BOOL_WIDTH),
         });
 
         let mut solver = ConstantSolver {
@@ -575,7 +583,7 @@ mod tests {
         assert_eq!(
             constants[res].inner,
             ConstantInner::Scalar {
-                width: 4,
+                width: crate::BOOL_WIDTH,
                 value: ScalarValue::Bool(true),
             },
         );
