@@ -28,7 +28,7 @@ pub enum Terminator {
     ///
     /// switch(SELECTOR) {
     ///  case TARGET_LITERAL#: {
-    ///    TARGET_BLOCK#  
+    ///    TARGET_BLOCK#
     ///  }
     ///  default: {
     ///    DEFAULT
@@ -38,7 +38,7 @@ pub enum Terminator {
         ///
         selector: Handle<crate::Expression>,
         /// Default block of the switch case.
-        default: BlockId,
+        default_id: BlockId,
         /// Tuples of (literal, target block)
         targets: Vec<(i32, BlockId)>,
     },
@@ -90,6 +90,7 @@ impl<I: Iterator<Item = u32>> super::Parser<I> {
                 },
                 local_variables: Arena::new(),
                 expressions: self.make_expression_storage(),
+                named_expressions: crate::FastHashMap::default(),
                 body: Vec::new(),
             }
         };
@@ -149,8 +150,8 @@ impl<I: Iterator<Item = u32>> super::Parser<I> {
                         fun_id,
                         &mut fun.expressions,
                         &mut fun.local_variables,
+                        &mut module.constants,
                         &module.types,
-                        &module.constants,
                         &module.global_variables,
                     )?;
 
@@ -181,7 +182,7 @@ impl<I: Iterator<Item = u32>> super::Parser<I> {
             }
         }
 
-        fun.body = flow_graph.to_naga()?;
+        fun.body = flow_graph.convert_to_naga()?;
 
         // done
         let fun_handle = module.functions.append(fun);
@@ -189,11 +190,12 @@ impl<I: Iterator<Item = u32>> super::Parser<I> {
         if let Some(ep) = self.lookup_entry_point.remove(&fun_id) {
             // create a wrapping function
             let mut function = crate::Function {
-                name: None,
+                name: Some(format!("{}_wrap", ep.name)),
                 arguments: Vec::new(),
                 result: None,
                 local_variables: Arena::new(),
                 expressions: Arena::new(),
+                named_expressions: crate::FastHashMap::default(),
                 body: Vec::new(),
             };
 
@@ -216,7 +218,7 @@ impl<I: Iterator<Item = u32>> super::Parser<I> {
                         let handle = function.expressions.append(crate::Expression::As {
                             expr: arg_expr,
                             kind: crate::ScalarKind::Sint,
-                            convert: true,
+                            convert: Some(4),
                         });
                         function.body.push(crate::Statement::Emit(
                             function.expressions.range_from(old_len),
@@ -353,9 +355,7 @@ impl<I: Iterator<Item = u32>> super::Parser<I> {
                     let ty = module.types.append(crate::Type {
                         name: None,
                         inner: crate::TypeInner::Struct {
-                            level: crate::StructLevel::Normal {
-                                alignment: crate::Alignment::new(1).unwrap(),
-                            },
+                            top_level: false,
                             members,
                             span: 0xFFFF, // shouldn't matter
                         },
