@@ -1,22 +1,108 @@
 /*! Universal shader translator.
 
-The central structure of the crate is [`Module`].
+The central structure of the crate is [`Module`]. A `Module` contains:
 
-To improve performance and reduce memory usage, most structures are stored
-in an [`Arena`], and can be retrieved using the corresponding [`Handle`].
+- [`EntryPoint`]s, the main functions for pipeline stages like vertex shading or
+  fragment shading,
 
-Functions are described in terms of statement trees. A statement may have
-branching control flow, and it may be mutating. Statements refer to expressions.
+- [`Function`]s, representing functions used by `EntryPoint`s and other `Function`s,
 
-Expressions form a DAG, without any side effects. Expressions need to be
-emitted in order to take effect. This happens in one of the following ways:
-  1. Constants and function arguments are implicitly emitted at function start.
-      This corresponds to `expression.needs_pre_emit()` condition.
-  2. Local and global variables are implicitly emitted. However, in order to use parts
-      of them in right-hand-side expressions, the `Expression::Load` must be explicitly emitted,
-      with an exception of `StorageClass::Handle` global variables.
-  3. Result of `Statement::Call` is automatically emitted.
-  4. `Statement::Emit` range is explicitly emitted.
+- [`Constant`]s and [`GlobalVariable`]s used by `EntryPoint`s and `Function`s, and
+
+- [`Type`]s used by the above.
+
+The body of an `EntryPoint` or `Function` is represented using two types:
+
+- An [`Expression`] produces a value, but has no side effects or control flow.
+  `Expressions` include variable references, unary and binary operators, and so
+  on.
+
+- A [`Statement`] can have side effects and structured control flow.
+  `Statement`s do not produce a value, other than by storing one in some
+  designated place. `Statements` include blocks, conditionals, and loops, but also
+  operations that have side effects, like stores and function calls.
+
+`Statement`s form a tree, with pointers into the DAG of `Expression`s.
+
+Restricting side effects to statements simplifies analysis and code generation.
+A Naga backend can generate code to evaluate an `Expression` however and
+whenever it pleases, as long as it is certain to observe the side effects of all
+previously executed `Statement`s.
+
+Many `Statement` variants use the [`Block`] type, which is simply `Vec<Block>`,
+representing a series of statements executed in order. The body of an
+`EntryPoint`s or `Function` is a `Block`, and `Statement` has a
+[`Block`][Statement::Block] variant.
+
+To improve translator performance and reduce memory usage, most structures are
+stored in an [`Arena`]. An `Arena<T>` stores a series of `T` values, indexed by
+[`Handle<T>`](Handle) values, which are just wrappers around integer indexes.
+For example, a `Function`'s expressions are stored in an `Arena<Expression>`,
+and compound expressions refer to their sub-expressions via `Handle<Expression>`
+values. (When examining the serialized form of a `Module`, note that the first
+element of an `Arena` has an index of 1, not 0.)
+
+## Function Calls
+
+The Naga IR's representation of function calls is unusual. Most languages treat
+function calls as expressions, but because calls may have side effects, Naga
+represents them with [`Statement::Call`]. A call statement may designate a
+particular `Expression` to represent its return value, if any, which can be used
+by subsequent statements and their expressions.
+
+## `Expression` evaluation time and scope
+
+While the order of execution of [`Statement`]s is apparent from their structure,
+it is not so obvious exactly when a given [`Expression`] should be evaluated,
+since many `Statement`s and `Expression`s may refer to its value. But it is
+essential to clearly specify an expression's evaluation time, since that
+determines which statements' effects the expression should observe. It is also
+helpful to backends to limit the visibility of an `Expression`'s value to a
+portion of the statement tree.
+
+An `Expression` may only be used, whether by a `Statement` or another
+`Expression`, if one of the following is true:
+
+- The expression is an [`Expression::Constant`], [`Expression::FunctionArgument`], or
+  [`Expression::GlobalVariable`].
+
+- The expression is an [`Expression::LocalVariable`] that is either the
+  `pointer` (destination) of a [`Statement::Store`], or initialized by some
+  previously executed `Statement::Store`.
+
+- The expression is the `result` of a [`Statement::Call`], representing the
+  call's return value. The call must be 'in scope' for the use (see below).
+
+- The expression is included in the range of some [`Statement::Emit`] that is
+  'in scope' for the use (see below). The [`Expression::needs_pre_emit`] method
+  returns `true` if the given expression does *not* need to be covered by an
+  `Emit` statement.
+
+The scope of an expression evaluated by an `Emit` statement covers the
+subsequent expressions in that `Emit`, any following statements in the `Block`
+to which that `Emit` belongs (if any) and their sub-statements (if any).
+
+If a `Call` statement has a `result` expression, then it is in scope for any
+statements following the `Call` in the `Block` to which that `Call` belongs (if
+any) and their sub-statements (if any).
+
+This means that, for example, an expression evaluated by some statement in a
+nested `Block` is not available in the `Block`'s parents. Such a value would
+need to be stored in a local variable to be carried upwards in the statement
+tree.
+
+## Variables
+
+An [`Expression::LocalVariable`] or [`Expression::GlobalVariable`] produces a
+pointer value referring to the variable's value. To retrieve the variable's
+value, use an [`Expression::Load`], with the variable expression as its
+`pointer` operand. To assign to a variable, use a [`Statement::Store`] with the
+variable expression as its `pointer` operand.
+
+As an exception, [`Expression::GlobalVariable`]s referring to
+[`GlobalVariable`]s whose `class` is [`StorageClass::Handle`] produce the
+variable's value directly; no `Load` is needed. These global variables refer to
+opaque values like samplers and images.
 
 !*/
 
@@ -50,7 +136,6 @@ pub use crate::arena::{Arena, Handle, Range};
 use std::{
     collections::{HashMap, HashSet},
     hash::BuildHasherDefault,
-    num::NonZeroU32,
 };
 
 #[cfg(feature = "deserialize")]
@@ -66,6 +151,9 @@ pub type FastHashMap<K, T> = HashMap<K, T, BuildHasherDefault<fxhash::FxHasher>>
 /// Hash set that is faster but not resilient to DoS attacks.
 pub type FastHashSet<K> = HashSet<K, BuildHasherDefault<fxhash::FxHasher>>;
 
+/// Map of expressions that have associated variable names
+pub(crate) type NamedExpressions = FastHashMap<Handle<Expression>, String>;
+
 /// Early fragment tests. In a standard situation if a driver determines that it is possible to
 /// switch on early depth test it will. Typical situations when early depth test is switched off:
 ///   - Calling ```discard``` in a shader.
@@ -150,6 +238,7 @@ pub enum BuiltIn {
     BaseInstance,
     BaseVertex,
     ClipDistance,
+    CullDistance,
     InstanceIndex,
     PointSize,
     VertexIndex,
@@ -168,7 +257,6 @@ pub enum BuiltIn {
 
 /// Number of bytes per scalar.
 pub type Bytes = u8;
-pub type Alignment = NonZeroU32;
 
 /// Number of components in a vector.
 #[repr(u8)]
@@ -357,18 +445,6 @@ pub enum ImageClass {
     Storage(StorageFormat),
 }
 
-/// Qualifier of the type level, at which a struct can be used.
-#[derive(Clone, Copy, Debug, Hash, Eq, Ord, PartialEq, PartialOrd)]
-#[cfg_attr(feature = "serialize", derive(Serialize))]
-#[cfg_attr(feature = "deserialize", derive(Deserialize))]
-pub enum StructLevel {
-    /// This is a root level struct, it can't be nested inside
-    /// other composite types.
-    Root,
-    /// This is a normal struct, and it has to be aligned for nesting.
-    Normal { alignment: Alignment },
-}
-
 /// A data type declared in the module.
 #[derive(Debug, PartialEq)]
 #[cfg_attr(feature = "serialize", derive(Serialize))]
@@ -419,8 +495,9 @@ pub enum TypeInner {
     },
     /// User-defined structure.
     Struct {
-        level: StructLevel,
+        top_level: bool,
         members: Vec<StructMember>,
+        //TODO: should this be unaligned?
         span: u32,
     },
     /// Possibly multidimensional array of texels.
@@ -444,7 +521,7 @@ pub struct Constant {
 }
 
 /// A literal scalar value, used in constants.
-#[derive(Debug, PartialEq, Clone, Copy, PartialOrd)]
+#[derive(Debug, Clone, Copy, PartialOrd)]
 #[cfg_attr(feature = "serialize", derive(Serialize))]
 #[cfg_attr(feature = "deserialize", derive(Deserialize))]
 pub enum ScalarValue {
@@ -485,7 +562,7 @@ pub enum Binding {
 }
 
 /// Pipeline binding information for global resources.
-#[derive(Clone, Debug, PartialEq)]
+#[derive(Clone, Debug, Hash, PartialEq)]
 #[cfg_attr(feature = "serialize", derive(Serialize))]
 #[cfg_attr(feature = "deserialize", derive(Deserialize))]
 pub struct ResourceBinding {
@@ -696,6 +773,19 @@ pub enum SwizzleComponent {
     W = 3,
 }
 
+bitflags::bitflags! {
+    /// Memory barrier flags.
+    #[cfg_attr(feature = "serialize", derive(Serialize))]
+    #[cfg_attr(feature = "deserialize", derive(Deserialize))]
+    #[derive(Default)]
+    pub struct Barrier: u32 {
+        /// Barrier affects all `StorageClass::Storage` accesses.
+        const STORAGE = 0x1;
+        /// Barrier affects all `StorageClass::WorkGroup` accesses.
+        const WORK_GROUP = 0x2;
+    }
+}
+
 /// An expression that can be evaluated to obtain a value.
 ///
 /// This is a Single Static Assignment (SSA) scheme similar to SPIR-V.
@@ -705,9 +795,58 @@ pub enum SwizzleComponent {
 #[cfg_attr(feature = "deserialize", derive(Deserialize))]
 pub enum Expression {
     /// Array access with a computed index.
+    ///
+    /// ## Typing rules
+    ///
+    /// The `base` operand must be some composite type: [`Vector`], [`Matrix`],
+    /// [`Array`], a [`Pointer`] to one of those, or a [`ValuePointer`] with a
+    /// `size`.
+    ///
+    /// The `index` operand must be an integer, signed or unsigned.
+    ///
+    /// Indexing a [`Vector`] or [`Array`] produces a value of its element type.
+    /// Indexing a [`Matrix`] produces a [`Vector`].
+    ///
+    /// Indexing a [`Pointer`] to an [`Array`] produces a [`Pointer`] to its
+    /// `base` type, taking on the `Pointer`'s storage class.
+    ///
+    /// Indexing a [`Pointer`] to a [`Vector`] produces a [`ValuePointer`] whose
+    /// size is `None`, taking on the [`Vector`]'s scalar kind and width and the
+    /// [`Pointer`]'s storage class.
+    ///
+    /// Indexing a [`Pointer`] to a [`Matrix`] produces a [`ValuePointer`] for a
+    /// column of the matrix: its size is the matrix's height, its `kind` is
+    /// [`Float`], and it inherits the [`Matrix`]'s width and the [`Pointer`]'s
+    /// storage class.
+    ///
+    /// ## Dynamic indexing restrictions
+    ///
+    /// To accommodate restrictions in some of the shader languages that Naga
+    /// targets, it is not permitted to subscript a matrix or array with a
+    /// dynamically computed index unless that matrix or array appears behind a
+    /// pointer. In other words, if the inner type of `base` is [`Array`] or
+    /// [`Matrix`], then `index` must be a constant. But if the type of `base`
+    /// is a [`Pointer`] to an array or matrix or a [`ValuePointer`] with a
+    /// `size`, then the index may be any expression of integer type.
+    ///
+    /// You can use the [`Expression::is_dynamic_index`] method to determine
+    /// whether a given index expression requires matrix or array base operands
+    /// to be behind a pointer.
+    ///
+    /// (It would be simpler to always require the use of `AccessIndex` when
+    /// subscripting arrays and matrices that are not behind pointers, but to
+    /// accommodate existing front ends, Naga also permits `Access`, with a
+    /// restricted `index`.)
+    ///
+    /// [`Vector`]: TypeInner::Vector
+    /// [`Matrix`]: TypeInner::Matrix
+    /// [`Array`]: TypeInner::Array
+    /// [`Pointer`]: TypeInner::Pointer
+    /// [`ValuePointer`]: TypeInner::ValuePointer
+    /// [`Float`]: ScalarKind::Float
     Access {
         base: Handle<Expression>,
-        index: Handle<Expression>, //int
+        index: Handle<Expression>,
     },
     /// Array access with a known index.
     AccessIndex {
@@ -777,6 +916,9 @@ pub enum Expression {
         right: Handle<Expression>,
     },
     /// Select between two values based on a condition.
+    ///
+    /// Note that, because expressions have no side effects, it is unobservable
+    /// whether the non-selected branch is evaluated.
     Select {
         /// Boolean expression
         condition: Handle<Expression>,
@@ -801,14 +943,15 @@ pub enum Expression {
         arg1: Option<Handle<Expression>>,
         arg2: Option<Handle<Expression>>,
     },
-    /// Cast a simply type to another kind.
+    /// Cast a simple type to another kind.
     As {
         /// Source expression, which can only be a scalar or a vector.
         expr: Handle<Expression>,
         /// Target scalar kind.
         kind: ScalarKind,
-        /// True = conversion needs to take place; False = bitcast.
-        convert: bool,
+        /// If provided, converts to the specified byte width.
+        /// Otherwise, bitcast.
+        convert: Option<Bytes>,
     },
     /// Result of calling another function.
     Call(Handle<Function>),
@@ -846,6 +989,10 @@ pub struct SwitchCase {
 #[cfg_attr(feature = "deserialize", derive(Deserialize))]
 pub enum Statement {
     /// Emit a range of expressions, visible to all statements that follow in this block.
+    ///
+    /// See the [module-level documentation][emit] for details.
+    ///
+    /// [emit]: index.html#expression-evaluation-time-and-scope
     Emit(Range<Expression>),
     /// A block containing more statements, to be executed sequentially.
     Block(Block),
@@ -871,6 +1018,10 @@ pub enum Statement {
     Return { value: Option<Handle<Expression>> },
     /// Aborts the current shader execution.
     Kill,
+    /// Synchronize invocations within the work group.
+    /// The `Barrier` flags control which memory accesses should be synchronized.
+    /// If empty, this becomes purely an execution barrier.
+    Barrier(Barrier),
     /// Stores a value at an address.
     ///
     /// This statement is a barrier for any operations on the
@@ -944,6 +1095,8 @@ pub struct Function {
     pub local_variables: Arena<LocalVariable>,
     /// Expressions used inside this function.
     pub expressions: Arena<Expression>,
+    /// Map of expressions that have associated variable names
+    pub named_expressions: NamedExpressions,
     /// Block of instructions comprising the body of the function.
     pub body: Block,
 }
@@ -975,7 +1128,7 @@ pub struct EntryPoint {
 /// To create a new module, use the `Default` implementation.
 /// Alternatively, you can load an existing shader using one of the [available front ends][front].
 ///
-/// When finished, you can export modules using one of the [available back ends][back].
+/// When finished, you can export modules using one of the [available backends][back].
 #[derive(Debug, Default)]
 #[cfg_attr(feature = "serialize", derive(Serialize))]
 #[cfg_attr(feature = "deserialize", derive(Deserialize))]
