@@ -111,6 +111,7 @@ pub enum ResolveError {
 
 pub struct ResolveContext<'a> {
     pub constants: &'a Arena<crate::Constant>,
+    pub types: &'a Arena<crate::Type>,
     pub global_vars: &'a Arena<crate::GlobalVariable>,
     pub local_vars: &'a Arena<crate::LocalVariable>,
     pub functions: &'a Arena<crate::Function>,
@@ -121,13 +122,21 @@ impl<'a> ResolveContext<'a> {
     pub fn resolve(
         &self,
         expr: &crate::Expression,
-        types: &'a Arena<crate::Type>,
         past: impl Fn(Handle<crate::Expression>) -> &'a TypeResolution,
     ) -> Result<TypeResolution, ResolveError> {
         use crate::TypeInner as Ti;
+        let types = self.types;
         Ok(match *expr {
             crate::Expression::Access { base, .. } => match *past(base).inner_with(types) {
+                // Arrays and matrices can only be indexed dynamically behind a
+                // pointer, but that's a validation error, not a type error, so
+                // go ahead provide a type here.
                 Ti::Array { base, .. } => TypeResolution::Handle(base),
+                Ti::Matrix { rows, width, .. } => TypeResolution::Value(Ti::Vector {
+                    size: rows,
+                    kind: crate::ScalarKind::Float,
+                    width,
+                }),
                 Ti::Vector {
                     size: _,
                     kind,
@@ -144,26 +153,39 @@ impl<'a> ResolveContext<'a> {
                     width,
                     class,
                 }),
-                Ti::Pointer { base, class } => TypeResolution::Value(match types[base].inner {
-                    Ti::Array { base, .. } => Ti::Pointer { base, class },
-                    Ti::Vector {
-                        size: _,
-                        kind,
-                        width,
-                    } => Ti::ValuePointer {
-                        size: None,
-                        kind,
-                        width,
-                        class,
-                    },
-                    ref other => {
-                        log::error!("Access sub-type {:?}", other);
-                        return Err(ResolveError::InvalidSubAccess {
-                            ty: base,
-                            indexed: false,
-                        });
-                    }
-                }),
+                Ti::Pointer { base, class } => {
+                    TypeResolution::Value(match types[base].inner {
+                        Ti::Array { base, .. } => Ti::Pointer { base, class },
+                        Ti::Vector {
+                            size: _,
+                            kind,
+                            width,
+                        } => Ti::ValuePointer {
+                            size: None,
+                            kind,
+                            width,
+                            class,
+                        },
+                        // Matrices are only dynamically indexed behind a pointer
+                        Ti::Matrix {
+                            columns: _,
+                            rows,
+                            width,
+                        } => Ti::ValuePointer {
+                            kind: crate::ScalarKind::Float,
+                            size: Some(rows),
+                            width,
+                            class,
+                        },
+                        ref other => {
+                            log::error!("Access sub-type {:?}", other);
+                            return Err(ResolveError::InvalidSubAccess {
+                                ty: base,
+                                indexed: false,
+                            });
+                        }
+                    })
+                }
                 ref other => {
                     log::error!("Access type {:?}", other);
                     return Err(ResolveError::InvalidAccess {
@@ -616,14 +638,21 @@ impl<'a> ResolveContext<'a> {
             crate::Expression::As {
                 expr,
                 kind,
-                convert: _,
+                convert,
             } => match *past(expr).inner_with(types) {
-                Ti::Scalar { kind: _, width } => TypeResolution::Value(Ti::Scalar { kind, width }),
+                Ti::Scalar { kind: _, width } => TypeResolution::Value(Ti::Scalar {
+                    kind,
+                    width: convert.unwrap_or(width),
+                }),
                 Ti::Vector {
                     kind: _,
                     size,
                     width,
-                } => TypeResolution::Value(Ti::Vector { kind, size, width }),
+                } => TypeResolution::Value(Ti::Vector {
+                    kind,
+                    size,
+                    width: convert.unwrap_or(width),
+                }),
                 ref other => {
                     return Err(ResolveError::IncompatibleOperands(format!(
                         "{:?} as {:?}",
