@@ -19,6 +19,8 @@ pub enum ExpressionError {
     InvalidIndexType(Handle<crate::Expression>),
     #[error("Accessing index {1} is out of {0:?} bounds")]
     IndexOutOfBounds(Handle<crate::Expression>, u32),
+    #[error("The expression {0:?} may only be indexed by a constant")]
+    IndexMustBeConstant(Handle<crate::Expression>),
     #[error("Function argument {0:?} doesn't exist")]
     FunctionArgumentDoesntExist(u32),
     #[error("Constant {0:?} doesn't exist")]
@@ -142,17 +144,16 @@ impl super::Validator {
 
         let stages = match *expression {
             E::Access { base, index } => {
-                match *resolver.resolve(base)? {
-                    Ti::Vector { .. }
-                    | Ti::Matrix { .. }
-                    | Ti::Array { .. }
-                    | Ti::Pointer { .. }
-                    | Ti::ValuePointer { size: Some(_), .. } => {}
+                // See the documentation for `Expression::Access`.
+                let dynamic_indexing_restricted = match *resolver.resolve(base)? {
+                    Ti::Vector { .. } => false,
+                    Ti::Matrix { .. } | Ti::Array { .. } => true,
+                    Ti::Pointer { .. } | Ti::ValuePointer { size: Some(_), .. } => false,
                     ref other => {
                         log::error!("Indexing of {:?}", other);
                         return Err(ExpressionError::InvalidBaseType(base));
                     }
-                }
+                };
                 match *resolver.resolve(index)? {
                     //TODO: only allow one of these
                     Ti::Scalar {
@@ -168,6 +169,11 @@ impl super::Validator {
                         return Err(ExpressionError::InvalidIndexType(index));
                     }
                 }
+                if dynamic_indexing_restricted
+                    && function.expressions[index].is_dynamic_index(module)
+                {
+                    return Err(ExpressionError::IndexMustBeConstant(base));
+                }
                 ShaderStages::all()
             }
             E::AccessIndex { base, index } => {
@@ -576,7 +582,15 @@ impl super::Validator {
                 let left_inner = resolver.resolve(left)?;
                 let right_inner = resolver.resolve(right)?;
                 let good = match op {
-                    Bo::Add | Bo::Subtract | Bo::Divide | Bo::Modulo => match *left_inner {
+                    Bo::Add | Bo::Subtract => match *left_inner {
+                        Ti::Scalar { kind, .. } | Ti::Vector { kind, .. } => match kind {
+                            Sk::Uint | Sk::Sint | Sk::Float => left_inner == right_inner,
+                            Sk::Bool => false,
+                        },
+                        Ti::Matrix { .. } => left_inner == right_inner,
+                        _ => false,
+                    },
+                    Bo::Divide | Bo::Modulo => match *left_inner {
                         Ti::Scalar { kind, .. } | Ti::Vector { kind, .. } => match kind {
                             Sk::Uint | Sk::Sint | Sk::Float => left_inner == right_inner,
                             Sk::Bool => false,
@@ -1103,14 +1117,30 @@ impl super::Validator {
                     .resolve(expr)?
                     .scalar_kind()
                     .ok_or(ExpressionError::InvalidCastArgument)?;
-                if !convert && prev_kind == Sk::Bool || kind == Sk::Bool {
-                    return Err(ExpressionError::InvalidCastArgument);
+                match convert {
+                    Some(width) if !self.check_width(kind, width) => {
+                        return Err(ExpressionError::InvalidCastArgument)
+                    }
+                    None if prev_kind == Sk::Bool || kind == Sk::Bool => {
+                        return Err(ExpressionError::InvalidCastArgument)
+                    }
+                    _ => {}
                 }
                 ShaderStages::all()
             }
             E::Call(function) => other_infos[function.index()].available_stages,
             E::ArrayLength(expr) => match *resolver.resolve(expr)? {
-                Ti::Array { .. } => ShaderStages::all(),
+                Ti::Pointer { base, .. } => {
+                    if let Some(&Ti::Array {
+                        size: crate::ArraySize::Dynamic,
+                        ..
+                    }) = resolver.types.try_get(base).map(|ty| &ty.inner)
+                    {
+                        ShaderStages::all()
+                    } else {
+                        return Err(ExpressionError::InvalidArrayType(expr));
+                    }
+                }
                 ref other => {
                     log::error!("Array length of {:?}", other);
                     return Err(ExpressionError::InvalidArrayType(expr));
