@@ -7,6 +7,7 @@ mod r#type;
 
 use crate::{
     arena::{Arena, Handle},
+    proc::{InvalidBaseType, Layouter},
     FastHashSet,
 };
 use bit_set::BitSet;
@@ -40,6 +41,26 @@ bitflags::bitflags! {
     }
 }
 
+impl Default for ValidationFlags {
+    fn default() -> Self {
+        Self::all()
+    }
+}
+
+#[must_use]
+bitflags::bitflags! {
+    /// Allowed IR capabilities.
+    #[derive(Default)]
+    #[cfg_attr(feature = "serialize", derive(serde::Serialize))]
+    #[cfg_attr(feature = "deserialize", derive(serde::Deserialize))]
+    pub struct Capabilities: u8 {
+        /// Support for `StorageClass:PushConstant`.
+        const PUSH_CONSTANT = 0x1;
+        /// Float values with width = 8.
+        const FLOAT64 = 0x2;
+    }
+}
+
 bitflags::bitflags! {
     /// Validation flags.
     #[cfg_attr(feature = "serialize", derive(serde::Serialize))]
@@ -69,7 +90,9 @@ impl ops::Index<Handle<crate::Function>> for ModuleInfo {
 #[derive(Debug)]
 pub struct Validator {
     flags: ValidationFlags,
+    capabilities: Capabilities,
     types: Vec<r#type::TypeInfo>,
+    layouter: Layouter,
     location_mask: BitSet,
     bind_group_masks: Vec<BitSet>,
     select_cases: FastHashSet<i32>,
@@ -91,6 +114,8 @@ pub enum ConstantError {
 
 #[derive(Clone, Debug, thiserror::Error)]
 pub enum ValidationError {
+    #[error(transparent)]
+    Layouter(#[from] InvalidBaseType),
     #[error("Type {handle:?} '{name}' is invalid")]
     Type {
         handle: Handle<crate::Type>,
@@ -170,10 +195,12 @@ impl crate::TypeInner {
 
 impl Validator {
     /// Construct a new validator instance.
-    pub fn new(flags: ValidationFlags) -> Self {
+    pub fn new(flags: ValidationFlags, capabilities: Capabilities) -> Self {
         Validator {
             flags,
+            capabilities,
             types: Vec::new(),
+            layouter: Layouter::default(),
             location_mask: BitSet::new(),
             bind_group_masks: Vec::new(),
             select_cases: FastHashSet::default(),
@@ -191,7 +218,7 @@ impl Validator {
         let con = &constants[handle];
         match con.inner {
             crate::ConstantInner::Scalar { width, ref value } => {
-                if !Self::check_width(value.scalar_kind(), width) {
+                if !self.check_width(value.scalar_kind(), width) {
                     return Err(ConstantError::InvalidType);
                 }
             }
@@ -224,6 +251,7 @@ impl Validator {
     /// Check the given module to be valid.
     pub fn validate(&mut self, module: &crate::Module) -> Result<ModuleInfo, ValidationError> {
         self.reset_types(module.types.len());
+        self.layouter.update(&module.types, &module.constants)?;
 
         if self.flags.contains(ValidationFlags::CONSTANTS) {
             for (handle, constant) in module.constants.iter() {
@@ -236,7 +264,6 @@ impl Validator {
             }
         }
 
-        // doing after the globals, so that `type_flags` is ready
         for (handle, ty) in module.types.iter() {
             let ty_info = self
                 .validate_type(handle, &module.types, &module.constants)
